import argparse
import json
import sys


# --------------------------------------------------------------------------- #
def parse_args():
    parser = argparse.ArgumentParser(
        prog=sys.argv[0].split('/')[-1],
        description="""
        \nHardy-Weinberg Equilibrium Calculator.
        \nCalculates the expected genotype frequencies based on the allele frequencies of a population 
        in Hardy-Weinberg equilibrium. 
        
        \nAlternate module usage: python3 -m src.main --ppop 10 --qpop 10 --pq2pop 200 --verbose
        
        \nSee: https://en.wikipedia.org/wiki/Hardy%E2%80%93Weinberg_principle""",
        usage='%(prog)s [-h] [--version] [--verbose] [--debug] [--samples SAMPLES] [--p P] [--q Q] [--tpop TPOP] ['
              '--ppop PPOP] [--qpop QPOP] [--pq2pop PQ2POP] [--genes GENES [GENES ...]]',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        epilog=("""\nExample: python3 -m %(prog)s --ppop 10 --qpop 10 --pq2pop 200 --verbose | """
                """\nExample: python3 -m [package].[module] --samples 1000 --verbose"""),
        add_help=True,
        allow_abbrev=True,
        exit_on_error=True,
    )

    parser.add_argument('--version', action='version', version='%(prog)s 0.1')
    parser.add_argument('--verbose', action='store_true', default=False,
                        help='Enable verbose logging.')
    parser.add_argument('--debug', action='store_true', default=False,
                        help='Enable debug logging.')
    parser.add_argument('--samples', type=int, default=None,

                        help='Number of samples to generate, if using random data generator.')
    parser.add_argument('--p', type=float, default=None,
                        help='Frequency of dominant allele.')
    parser.add_argument('--q', type=float, default=None,
                        help='Frequency of recessive allele.')
    parser.add_argument('--tpop', type=float, default=None,
                        help='Total population.')
    parser.add_argument('--ppop', type=float, default=None,
                        help='Original population of dominant allele.')
    parser.add_argument('--qpop', type=float, default=None,
                        help='Original population of recessive allele.')
    parser.add_argument('--pq2pop', type=float, default=None,
                        help='Original population of heterozygous allele.')
    parser.add_argument('--genes', type=json.loads, nargs='+', default=lambda o: o.__dict__(),
                        help='List of JSON[Genes] to calculate. Note: This is list of json objects representing of '
                             'Gene class.')

    return parser
# --------------------------------------------------------------------------- #
